@@ -1,7 +1,3 @@
-# Qualitative Coding corpus
-# -------------------------
-# (c) 2023 Chris Proctor
-
 from itertools import chain, combinations_with_replacement
 from collections import defaultdict
 from contextlib import contextmanager
@@ -10,6 +6,8 @@ from pathlib import Path
 import yaml
 import shutil
 import numpy as np
+from semver import Version
+import structlog
 from textwrap import fill
 import os
 from hashlib import sha1
@@ -32,7 +30,6 @@ from sqlalchemy.orm import (
 from sqlalchemy.dialects.sqlite import insert
 from qualitative_coding.helpers import prompt_for_choice
 from qualitative_coding.tree_node import TreeNode
-from qualitative_coding.logs import get_logger
 from qualitative_coding.exceptions import (
     QCError, 
     SettingsError, 
@@ -54,17 +51,28 @@ from qualitative_coding.helpers import (
     iter_paragraph_lines,
     read_settings,
 )
+from qualitative_coding.diff import (
+    get_diff, 
+    get_git_diff,
+    reindex_coded_lines,
+    in_git_repo,
+)
+
+log = structlog.get_logger()
 
 DEFAULT_SETTINGS = {
     'qc_version': metadata('qualitative-coding')['version'],
     'corpus_dir': 'corpus',
     'database': 'qualitative_coding.sqlite3',
-    'logs_dir': 'logs',
     'memos_dir': 'memos',
     'codebook': 'codebook.yaml',
     'editor': 'code',
+    'log_file': 'qualitative_coding.log',
+    'verbose': False,
 }
 
+LATEST_MIGRATION = Version.parse("1.4.0")
+
 class QCCorpus:
     """Provides data access to the corpus of documents and codes. 
     QCCorpus methods which access the database must be called from within
@@ -78,7 +86,7 @@ class QCCorpus:
     units = ["line", "paragraph", "document"]
 
     @classmethod
-    def initialize(cls, settings_path="settings.yaml", accept_defaults=False):
+    def initialize(cls, settings_path="settings.yaml", only_write_settings_file=False):
         """
         Initializes a qc project.
         If the settings file does not exist, create it.
@@ -86,33 +94,33 @@ class QCCorpus:
         directories and files.
         """
         settings_path = Path(settings_path)
-        if settings_path.exists() or accept_defaults:
+        if only_write_settings_file:
+            if settings_path.exists():
+                raise QCError(f"Settings file {settings_path} already exists.")
+            settings_path.write_text(yaml.dump(DEFAULT_SETTINGS))
+        else:
             if not settings_path.exists():
                 settings_path.write_text(yaml.dump(DEFAULT_SETTINGS))
+            project_root = settings_path.parent
             cls.validate_settings(settings_path)
             settings = yaml.safe_load(settings_path.read_text())
-            for needed_dir in ["corpus_dir", "logs_dir", "memos_dir"]:
-                dirpath = Path(settings[needed_dir])
+            for needed_dir in ["corpus_dir", "memos_dir"]:
+                dirpath = project_root / Path(settings[needed_dir])
                 if dirpath.exists() and not dirpath.is_dir():
                     raise QCError(f"Cannot create dir {dirpath}; there is a file with that name.")
                 if not dirpath.exists():
                     dirpath.mkdir()
-            codebook_path = Path(settings["codebook"])
+            codebook_path = project_root / Path(settings["codebook"])
             if codebook_path.exists() and codebook_path.is_dir():
-                raise QCError(f"Cannot create {dirpath}; there is a directory with that name.")
+                raise QCError(f"Cannot create {codebook_path}; there is a directory with that name.")
             if not codebook_path.exists():
                 codebook_path.touch()
-            if Path(settings["database"]).is_absolute():
-                db_path = Path(settings["database"])
-            else:
-                db_path = settings_path.resolve().parent / settings["database"]
-            if db_path.exists() and db_path.is_dir():
-                raise QCError(f"Cannot create {dirpath}; there is a directory with that name.")
-            if not db_path.exists():
-                engine = create_engine(f"sqlite:///{db_path}")
+            database_path = project_root / Path(settings["database"])
+            if database_path.exists() and database_path.is_dir():
+                raise QCError(f"Cannot create {database_path}; there is a directory with that name.")
+            if not database_path.exists():
+                engine = create_engine(f"sqlite:///{database_path}")
                 Base.metadata.create_all(engine)
-        else:
-            settings_path.write_text(yaml.dump(DEFAULT_SETTINGS))
 
     @classmethod
     def validate_settings(cls, settings_path):
@@ -126,11 +134,20 @@ class QCCorpus:
                 errors.append(f"Settings file {settings_path} is missing")
                 raise SettingsError()
             settings = read_settings(settings_path)
+            if not "qc_version" in settings:
+                log.warning("qc_version not found in settings file. Cannot validate settings.")
+                return
+            try:
+                version = Version.parse(settings["qc_version"])
+            except ValueError:
+                errors.append(f"could not read qc_version {settings['qc_version']}")
+                raise SettingsError()
+            if not version >= LATEST_MIGRATION:
+                log.warning("Cannot validate settings; project is out of date. Run qc upgrade.")
+                return
             for expected_key in DEFAULT_SETTINGS:
                 if expected_key not in settings:
                     errors.append(f"Expected '{expected_key}' in settings")
-                elif not isinstance(settings[expected_key], str):
-                    errors.append(f"Invalid path for {expected_key}: {settings[expected_key]}")
             if 'editors' in settings:
                 if not isinstance(settings['editors'], dict):
                     errors.append(f"Could not read custom-defined editors.")
@@ -151,6 +168,8 @@ class QCCorpus:
         except SettingsError:
             errfmt = [f" - {err}" for err in errors]
             raise QCError('\n'.join(["Error validating settings:"] + errfmt))
+        finally:
+            log.debug("Validating settings", errors=errors, settings_path=str(settings_path))
 
     def __init__(self, settings_path, skip_validation=False):
         self.settings_path = Path(settings_path)
@@ -159,8 +178,6 @@ class QCCorpus:
             self.validate()
         self.corpus_dir = self.resolve_path(self.settings['corpus_dir'])
         self.memos_dir = self.resolve_path(self.settings['memos_dir'])
-        self.logs_dir = self.resolve_path(self.settings['logs_dir'])
-        self.log = get_logger(__name__, self.logs_dir, self.settings.get('debug'))
         self.codebook_path = self.resolve_path(self.settings['codebook'])
         db_file = self.resolve_path(self.settings['database'])
         self.engine = create_engine(f"sqlite:///{db_file}")
@@ -212,7 +229,7 @@ class QCCorpus:
         "Checks that project state is valid"
         QCCorpus.validate_settings(self.settings_path)
         errors = []
-        for expected_dir in ['corpus_dir', 'logs_dir', 'memos_dir']:
+        for expected_dir in ['corpus_dir', 'memos_dir']:
             path = self.resolve_path(self.settings[expected_dir])
             if not path.exists():
                 errors.append(f"{expected_dir} path {path} does not exist")
@@ -260,7 +277,7 @@ class QCCorpus:
                         f"{doc.file_path} has been changed since it was imported. " + 
                         f"This could affect the alignment of existing codes. " + 
                         f"Either restore the original version of {doc.file_path}, or " + 
-                        f"import the changed version by running: qc corpus rebase " + 
+                        f"import the changed version by running: qc corpus update " + 
                         f"{doc.file_path}"
                     )
         if errors:
@@ -348,8 +365,8 @@ class QCCorpus:
     def get_document(self, corpus_path):
         """Fetches a document object.
         """
-        relpath = self.get_relative_corpus_path(corpus_path)
-        q = select(Document).where(Document.file_path == relpath)
+        relpath = self.get_corpus_path(corpus_path)
+        q = select(Document).where(Document.file_path == str(relpath))
         return self.get_session().scalars(q).first()
 
     def get_paragraph(self, document, line, index_name="paragraphs"):
@@ -359,7 +376,7 @@ class QCCorpus:
             select(Location)
             .join(Location.document_index)
             .join(DocumentIndex.document)
-            .where(Document.file_path == document.file_path)
+            .where(Document.file_path == document)
             .where(DocumentIndex.name == index_name)
             .where(Location.start_line <= line)
             .where(Location.end_line > line)
@@ -367,12 +384,12 @@ class QCCorpus:
         try:
             return self.get_session().execute(q).scalar_one()
         except NoResultFound:
-            raise QCError(f"Error getting paragraph for {document.file_path}, line {line}.")
+            raise QCError(f"Error getting paragraph for {document}, line {line}.")
 
     def update_coded_lines(self, document, coder, coded_line_data):
         """Updates document's coded lines for the given coder.
         document and coder should be strings, and coded_line_data should
-        be a list of dicts like {'line': 1, 'code_id': 'super}.
+        be a list of dicts like {'line': 1, 'code_id': 'super'}.
 
         Fetches all existing coded lines for the document and coder, and 
         then compares the set of existing coded line data with new coded line data.
@@ -386,7 +403,7 @@ class QCCorpus:
         q = (select(CodedLine)
             .join(CodedLine.locations)
             .join(Location.document_index)
-            .where(DocumentIndex.document_id == document.file_path)
+            .where(DocumentIndex.document_id == document)
             .where(CodedLine.coder_id == coder)
         )
         existing_coded_lines = self.session.scalars(q).all()
@@ -402,22 +419,6 @@ class QCCorpus:
         session.commit()
         self.update_codebook()
 
-    def get_relative_corpus_path(self, corpus_path):
-        """Given a Path or str, tries to return a string representation of the
-        path relative to the corpus directory, suitable for Document.file_path.
-        """
-        try:
-            relative_path = Path(corpus_path).relative_to(self.corpus_dir)
-            return str(relative_path)
-        except ValueError:
-            if Path(corpus_path).is_absolute():
-                raise ValueError(
-                    f"{corpus_path} is absolute and is not relative to corpus directory "
-                    f"{self.settings['corpus_dir']}"
-                )
-            else:
-                return str(corpus_path)
-
     def import_media(self, file_path, recursive=False, corpus_root=None, importer="pandoc"):
         """Imports media into the corpus. 
         Importing media consists of three tasks: 
@@ -476,9 +477,9 @@ class QCCorpus:
         doc = self.get_document(corpus_path)
         if doc:
             raise Document.AlreadyExists(doc)
-        relpath = self.get_relative_corpus_path(corpus_path)
+        relpath = self.get_corpus_path(corpus_path)
         document = Document(
-            file_path=relpath,
+            file_path=str(relpath),
             file_hash=self.hash_file(corpus_path),
         )
         self.get_session().add(document)
@@ -496,6 +497,18 @@ class QCCorpus:
                 ))
         self.get_session().commit()
 
+    def get_updated_coded_lines(self, file_path, diff):
+        """Returns [(code, coder, line, file_path)] after applying a file diff.
+        When a document is updated, its line numbers may change and consequently
+        existing paragraphs and existing coded lines may need to be re-indexed.
+        """
+        corpus_path = self.get_corpus_path(file_path)
+        coded_lines = self.get_coded_lines(file_list=[corpus_path])
+        log.debug(coded_lines)
+        # Get all existing coded lines. 
+        # Iterate through the diff
+        # return a list of updated coded lines.
+
     def count_codes(self, pattern=None, file_list=None, coders=None, unit="line"):
         """Returns a dict of {code:count}.
         """
@@ -524,42 +537,53 @@ class QCCorpus:
         """Move a file from target to destination, updating the database.
         """
         self.validate_corpus_paths()
-        target = self.corpus_dir / target
-        destination = self.corpus_dir / destination
-        if not target.exists():
-            raise QCError(f"{target} does not exist")
-        if destination.exists():
-            raise QCError(f"{destination} already exists")
-        if recursive and not target.is_dir():
+        target = self.get_corpus_path(target)
+        destination = self.get_corpus_path(destination)
+        if recursive and not (self.corpus_dir / target).is_dir():
             raise QCError(f"Cannot use --recursive when {target} is a file.")
-        if not recursive and target.is_dir():
+        if not recursive and (self.corpus_dir / target).is_dir():
             raise QCError(f"{target} is a directory. Use --recursive")
 
         if recursive:
-            for dir_path, dir_names, filenames in os.walk(target):
+            for dir_path, dir_names, filenames in os.walk(self.corpus_dir / target):
                 for fn in filenames:
-                    dp = Path(dir_path).relative_to(target)
+                    dp = Path(dir_path).relative_to(self.corpus_dir / target)
                     rtarget = target / dp / fn
                     rdestination = destination / dp / fn
-                    old_file_path = str(rtarget.relative_to(self.corpus_dir))
-                    new_file_path = str(rdestination.relative_to(self.corpus_dir))
+                    old_file_path = self.get_corpus_path(self.corpus_dir / rtarget)
+                    new_file_path = self.get_corpus_path(self.corpus_dir / rdestination)
                     self._move_document(old_file_path, new_file_path)
         else:
-            old_file_path = str(target.relative_to(self.corpus_dir))
-            new_file_path = str(destination.relative_to(self.corpus_dir))
-            self._move_document(old_file_path, new_file_path)
-        destination.parent.mkdir(parents=True, exist_ok=True)
-        target.rename(destination)
+            self._move_document(target, destination)
+        (self.corpus_dir / destination).parent.mkdir(parents=True, exist_ok=True)
+        (self.corpus_dir / target).rename(self.corpus_dir / destination)
         self.get_session().commit()
 
     def _move_document(self, old_file_path, new_file_path):
         """Updates a document's file path, and updates DocumentIndex.document_id
         to match. Does not commit the session.
         """
-        doc = self.get_documents(file_list=[old_file_path])[0]
+        doc = self.get_documents(file_list=[str(old_file_path)])[0]
         for document_index in doc.indices:
-            document_index.document_id = new_file_path
-        doc.file_path = new_file_path
+            document_index.document_id = str(new_file_path)
+        doc.file_path = str(new_file_path)
+
+    def get_corpus_path(self, target, must_exist=False, must_not_exist=False, 
+                must_be_file=False, must_be_dir=False):
+        """Returns target as a path relative to the corpus root.
+        """
+        target = Path(target).resolve()
+        if not target.is_relative_to(self.corpus_dir):
+            raise QCError(f"{target} is not a path within the corpus directory")
+        if (must_exist or must_be_file or must_be_dir) and not target.exists():
+            raise QCError(f"{target} does not exist")
+        if must_not_exist and target.exists():
+            raise QCError(f"{target} exists")
+        if must_be_file and not target.is_file():
+            raise QCError(f"{target} is not a file")
+        if must_be_dir and not target.is_dir():
+            raise QCError(f"{target} is not a directory")
+        return target.relative_to(self.corpus_dir)
 
     def remove_document(self, target, recursive=False):
         """Remove a document, or recursively remove all documents in a directory.
@@ -567,7 +591,7 @@ class QCCorpus:
         """
         session = self.get_session()
         self.validate_corpus_paths()
-        target = self.corpus_dir / target
+        target = self.corpus_dir / self.get_corpus_path(target)
         if not target.exists():
             raise QCError(f"{target} does not exist")
         if recursive and not target.is_dir():
@@ -598,7 +622,16 @@ class QCCorpus:
         Note that it does not commit the session.
         """
         session = self.get_session()
-        for cl in self.get_coded_lines(file_list=[file_path]):
+
+        query = (
+            select(CodedLine)
+            .join(CodedLine.locations)
+            .join(Location.document_index)
+            .where(DocumentIndex.name == "paragraphs")
+        )
+        query = self.filter_query_by_document(query, None, [file_path])
+        coded_lines = self.get_session().execute(query).scalars()
+        for cl in coded_lines:
             session.delete(cl)
         doc = self.get_documents(file_list=[file_path])[0]
         session.delete(doc)
@@ -641,7 +674,7 @@ class QCCorpus:
         }[unit]
 
     def get_coded_lines(self, codes=None, pattern=None, file_list=None, coders=None):
-        """Returns (code, coder, line, file_path)
+        """Returns [(code, coder, line, file_path)]
         """
         query = (
             select(
@@ -855,13 +888,39 @@ class QCCorpus:
                 totals_by_doc[doc][node.expanded_name()] = getattr(node, attr)
         return totals_by_doc
 
+    def update_document(self, file_path, new, dryrun=False):
+        """Update the text of a corpus document. 
+        In addition to updating the text in the file, the hash in the database
+        needs to be updated and all existing coded lines need to be reindexed. 
+        """
+        if new:
+            log.debug("Using new file comparison diff strategy")
+            if not Path(new).exists():
+                raise InvalidParameter(f"new path {new} does not exist")
+            diff = get_diff(file_path, new)
+        else:
+            log.debug("Using git diff strategy")
+            if not in_git_repo():
+                raise QCError("update with git strategy can only be used within a git repository")
+            diff = get_git_diff(file_path)
 
-
-
-
-
-
-
-
-
-
+        if dryrun:
+            print(diff)
+        else:
+            corpus_path = str(self.get_corpus_path(file_path))
+            coded_lines = self.get_coded_lines(file_list=[corpus_path])
+            reindexed_coded_lines = reindex_coded_lines(coded_lines, diff)
+            coded_lines_by_file_by_coder = defaultdict(lambda: defaultdict(list))
+            for code, coder, line, file_path in reindexed_coded_lines:
+                coded_lines_by_file_by_coder[file_path][coder].append({
+                    'line': line, 
+                    'code_id': code,
+                })
+            for file_path, lines_by_coder in coded_lines_by_file_by_coder.items():
+                for coder, lines in lines_by_coder.items():
+                    self.update_coded_lines(file_path, coder, lines)
+            if new:
+                (self.corpus_dir / corpus_path).write_text(Path(new).read_text())
+            doc = self.get_document(self.corpus_dir / corpus_path)
+            doc.file_hash = self.hash_file(self.corpus_dir / corpus_path)
+            self.get_session().commit()
