@@ -2,15 +2,18 @@ from tests.fixtures import QCTestCase
 from pathlib import Path
 from qualitative_coding.corpus import DEFAULT_SETTINGS
 
-class TestCode(QCTestCase):
+class TestCheck(QCTestCase):
     def test_check_passes_when_no_errors(self):
         result = self.run_in_testpath("qc check")
         self.assertEqual(result.stdout, "")
 
     def test_check_identifies_missing_settings(self):
         for setting in DEFAULT_SETTINGS:
+            if setting == 'qc_version': 
+                continue
             self.update_settings(setting, None)
-            message = self.run_in_testpath("qc check").stderr
+            result = self.run_in_testpath("qc check")
+            message = result.stderr
             self.assertTrue(f"Expected '{setting}' in settings" in message)
 
     def test_check_validates_corpus_paths(self):
@@ -24,7 +27,7 @@ class TestCode(QCTestCase):
     def test_check_validates_corpus_paths(self):
         self.run_in_testpath("qc corpus import macbeth.txt")
         (self.testpath / "corpus" / "macbeth.txt").write_text("It was the best of times...")
-        message = self.run_in_testpath("qc check", debug=True).stderr
+        message = self.run_in_testpath("qc check").stderr
         self.assertTrue("macbeth.txt" in message)
 
 
